@@ -1,8 +1,6 @@
-version = (0, 1, 3)
-__version__ = '.'.join(map(str, version))
-
 from suds.client import Client
 from suds.xsd.doctor import ImportDoctor, Import
+import difflib
 
 class Status(object):
     AWAITING_COMPILATION = -1
@@ -19,15 +17,15 @@ class Result(object):
     MEMORY_LIMIT_EXCEEDED = 17
     ILLEGAL_SYSTEM_CALL = 19
     INTERNAL_ERROR = 20
-    
+
 
 class IdeoneError(Exception):
     pass
-    
+
 class Ideone(object):
 
     ERROR_OK = 'OK'
-    
+
     def __init__(self, user, password):
         self.user = user
         self.password = password
@@ -72,7 +70,7 @@ class Ideone(object):
             language_dict[key] = value
 
         return language_dict
-        
+
     def _translate_language_name(self, language_name):
         """
         Translate a human readable langauge name into its Ideone
@@ -90,7 +88,7 @@ class Ideone(object):
 
         Notes
         -----
-        
+
         We use a local cache of languages if available, else we grab
         the list of languages from Ideone.  We test for a string match
         by comparing prefixes because Ideone includes the language
@@ -99,37 +97,58 @@ class Ideone(object):
 
         Examples
         --------
-        
+
         >>> ideone_object = Ideone('username', 'password')
         >>> ideone_object._translate_language_name('ada')
         7
 
         """
 
-        if self._language_dict is None:
-            # TODO: Relying on the cache side effect.
-            self.languages()
+        languages = self.languages()
+        language_id = None
 
-        language_id = None        
-        for k,v in self._language_dict.items():
-            # Find it by comparing prefixes because the languages
-            # include version information
-            if v.lower().startswith(language_name.lower()):
-                language_id = k
+        # Check for exact match first including the whole version
+        # string
+        for ideone_index, ideone_language in languages.items():
+            if ideone_language.lower() == language_name.lower():
+                return ideone_index
 
-        return language_id
-            
+        # Check for a match of just the language name without any
+        # version information
+        simple_languages = dict((k,v.split('(')[0].strip())
+                                for (k,v) in languages.items())
+        for ideone_index, simple_name in simple_languages.items():
+            if simple_name.lower() == language_name.lower():
+                return ideone_index
+
+        # Give up, but first find a similar name, suggest it and error
+        # out
+        language_choices = languages.values() + simple_languages.values()
+        similar_choices = difflib.get_close_matches(language_name,
+                                                    language_choices,
+                                                    n=3,
+                                                    cutoff=0.3)
+        # Add quotes and delimit with strings for easier to read
+        # output
+        similar_choices_string = ", ".join(["'" + s + "'"
+                                            for s in similar_choices])
+        error_string = ("Couldn't match '%s' to an Ideone accepted language.\n"
+                        "Did you mean one of the following: %s")
+        raise IdeoneError(error_string % (language_name, similar_choices_string))
         
-    def create_submission(self, source_code, language_name, std_input="",
-                          run=True, private=False):
+
+
+    def create_submission(self, source_code, language_name=None, language_id=None,
+                          std_input="", run=True, private=False):
         """
         Create a submission and upload it to Ideone.
 
         Keyword Arguments
         -----------------
-        
+
         * source_code: a string of the programs source code
         * language_name: the human readable language string (e.g. 'python')
+        * language_id: the ID of the programming language
         * std_input: the string to pass to the program on stdin
         * run: a boolean flag to signifying if Ideone should compile and
                run the program
@@ -137,21 +156,21 @@ class Ideone(object):
 
         Returns
         -------
-        
+
         A dictionary with the keys error and link.  The link is the
         unique id of the program.  The URL of the submission is
         http://ideone.com/LINK.
 
         Examples
         --------
-        
+
         >>> ideone_object = Ideone('username', 'password')
-        >>> ideone_object.create_submission('print(42)', 'python')
+        >>> ideone_object.create_submission('print(42)', language_name='python')
         {'error': 'OK',
          'link' : 'LsSbo'}
 
         """
-        language_id = self._translate_language_name(language_name)
+        language_id = language_id or self._translate_language_name(language_name)
         result = self.client.service.createSubmission(self.user, self.password,
                                                       source_code, language_id,
                                                       std_input, run, private)
@@ -179,14 +198,14 @@ class Ideone(object):
         -----
 
         Status specifies the stage of execution.
-        
+
         * status < 0 means the program awaits compilation
         * status == 0 means the program is done
         * status == 1 means the program is being compiled
         * status == 3 means the program is running
 
         Result specifies how the program finished.
-        
+
         * result == 0 means not running, the program was submitted
                       with run=False
         * result == 11 means compilation error
@@ -196,12 +215,12 @@ class Ideone(object):
         * result == 17 means memory limit exceeded
         * result == 19 means illegal system call
         * result == 20 means Ideone internal error, submit a bug report
-        
-        
+
+
 
         Examples
         --------
-        
+
         >>> ideone_object = Ideone('username', 'password')
         >>> ideone_object.submission_status('LsSbo')
         {'error': 'OK',
@@ -230,7 +249,7 @@ class Ideone(object):
         * with_output: request the program output
         * with_stderr: request the error output
         * with_compilation_info: request compilation flags
-        
+
         Examples
         --------
 
@@ -244,7 +263,7 @@ class Ideone(object):
          'langName': "Python 3",
          'langVersion': "python-3.1.2",
          'memory': 5852,
-         'output': 42,        
+         'output': 42,
          'public': True,
          'result': 15,
          'signal': 0,
@@ -252,7 +271,7 @@ class Ideone(object):
          'status': 0,
          'stderr': "",
          'time': 0.02}
-        
+
         """
         result = self.client.service.getSubmissionDetails(self.user, self.password,
                                                           link,
@@ -269,7 +288,7 @@ class Ideone(object):
 
         Examples
         --------
-        
+
         >>> ideone_object.languages()
         {'error': 'OK',
          'languages': {1: "C++ (gcc-4.3.4)",
@@ -280,13 +299,14 @@ class Ideone(object):
                        125: "Falcon (falcon-0.9.6.6)"}}
 
         """
-        result = self.client.service.getLanguages(self.user, self.password)
-        result_dict = Ideone._transform_to_dict(result)
-        Ideone._handle_error(result_dict)
-        languages = result_dict['languages']
-        result_dict['languages'] = Ideone._collapse_language_array(languages)
-        self._language_dict = result_dict['languages']
-        return result_dict
+        if self._language_dict is None:
+            result = self.client.service.getLanguages(self.user, self.password)
+            result_dict = Ideone._transform_to_dict(result)
+            Ideone._handle_error(result_dict)
+            languages = result_dict['languages']
+            result_dict['languages'] = Ideone._collapse_language_array(languages)
+            self._language_dict = result_dict['languages']
+        return self._language_dict
 
     def test(self):
         """
